function [y1, y2]= load_data( inp_type, a1, a2, a3)
%
% function [y1, y2]= load_data( inp_type, a1, a2, a3)
%
% Load a mirror profile from two alternative types of data
% files or from the workspace. Returns a structure with the
% profile read.
%
% inp_type: 1x1 : 0 => mat file, 1 => txt file, 2 => arguments
%
% if inp_type==0 then a1= mat_filename
% if inp_type==1 then a1= txt_filename
%                     a2= params_filename [facultative]
%                     a3= log_polar_sensor_flag [facultative]
% if inp_type==2 then a1= array Nx2 with mirror points
%                 and a2= camera_to_mirror_offset
%
% For a detailed explanation about the contents of the various
% input modes see the help of mirror_vfield_gui.m .
%
% y1: Nx2 : mirror points
% y2: 1x1 : camera to mirror offset
%
% If there is only one output argument, then y1 becomes
% a structure with both the mirror points and the camera
% to mirror offset value.
%
% See also: MIRROR_VFIELD_GUI, MIRROR_VFIELD
%

%
%   Instituto Superior Tcnico / Instituto de Sistemas e Robtica
%           Computer Vision Lab: OMNIVIEWS PROJECT
%           http://www.isr.ist.utl.pt/labs/vislab/
%
% 10/10/01, J. Gaspar (jag@isr.ist.utl.pt)
%

switch inp_type
case 0,
   [xz_mirror, camera_to_mirror_offset]= load_mat_file( a1 );
   
case 1,
   if nargin<4, a3=[]; end
   if nargin<3, a2=[]; end
   [xz_mirror, camera_to_mirror_offset]= load_txt_file( a1, a2, a3 );
   
case 2,
   if size(a1,2)~=2,
      error(['"' inputname(2) '" has not 2 columns.'])
   end
   xz_mirror= [a1(:,1) -a1(:,2)];
   camera_to_mirror_offset= a2;
   
otherwise,
   error(['invalid inp_type value: ' num2str(inp_type)])
end

if nargout>1,
   y1= xz_mirror;
   y2= camera_to_mirror_offset;
else
   y1= struct('xz_mirror',xz_mirror, ...
      'camera_to_mirror_offset',camera_to_mirror_offset);
end

return; % end of function


% ----------------- aux fn:
%
function [xz_mirror, camera_to_mirror_offset]= load_mat_file( mat_fname )
%
load(mat_fname);
if ~exist('xz_mirror','var'),
   error(['file "' mat_fname '" does not contain the matrix "xz_mirror"'])
end
if ~exist('camera_to_mirror_offset','var'),
   error(['file "' mat_fname '" does not contain the matrix "camera_to_mirror_offset"'])
end
%
xz_mirror= [xz_mirror(:,1) -xz_mirror(:,2)];
%
return; % end of function


% ----------------- aux fn:
%
function [xz_mirror, camera_to_mirror_offset]= ...
   load_txt_file( txt_fname, params_fname, log_polar_ccd_flag )

if isempty(log_polar_ccd_flag),
   %
   % try to figure out from txt_fname if it is a mirror
   % for a linear or for a log-polar camera
   %
   log_polar_ccd_flag= 0;
   if ~isempty(findstr(txt_fname,'_log')),
      log_polar_ccd_flag= 1;
   end
end

if isempty(params_fname),
   %
   % try to figure out the parameters filename
   %
   if log_polar_ccd_flag,
      params_fname= strrep(txt_fname, 'log.txt', 'params.m');
   else
      params_fname= strrep(txt_fname, 'lin.txt', 'params.m');
   end
end

% load array of points
%
xz_mirror= load(txt_fname);

% load parameters
%
eval( txtFile_to_mtlbStr(params_fname) );

if exist('camera_to_mirror_offset','var'),
   %
   % work done: no need to calculate camera_to_mirror_offset
   %
   return;
end

% define camera to mirror offset (the focal length must
% be removed since the original data was defined in relation
% to the image plane).
%
if log_polar_ccd_flag,
   F= (lastFValue_log -f_logpcamera)*10; % convert to mm
else
   F= (lastFValue_lin -f_convcamera)*10; % convert to mm
end
camera_to_mirror_offset= F;

return; % end of function


% ---------- aux fn:
%
function str= txtFile_to_mtlbStr(fname)
%
fid= fopen(fname, 'rt');
if fid==-1,
   error(['opening file: ' fname]);
end
%
str= [];
while 1
   line = fgetl(fid);
   if ~isstr(line), break, end
   str= [str '; ' line];
end
fclose(fid);
%
return; % end of function
