function mirror_vfield(xz, camCentre2Mirror)
%
% Show mirror profile and view field
%
% Usage:
%   mirror_vfield(xz, camera_to_mirror_offset)
%
% xz : Nx2 : mirror points; each line constains radius and z; z values will
%            be tipically negative
% camCentre2Mirror: 1x1: distance from the camera to the mirror along the
%            optical axis. If the mirror points have an offset accounting for
%            the camera distance, than camCentre2Mirror must be specified as zero.
%
% Co-ordinate system expected for XZ values:
%
%      \    |    /
%       \   |   /
%         . | .
%     ------+-------> x
%           |O
%           |                                  --
%           |         camera_to_mirror_offset= CO
%           |
%           |
%           + C
%           |
%           v
%            z
%

%
%   Instituto Superior Tcnico / Instituto de Sistemas e Robtica
%           Computer Vision Lab: OMNIVIEWS PROJECT
%           http://www.isr.ist.utl.pt/labs/vislab/
%
% 09/05/01, 25/06/01, 04/07/01,
% 09/10/01 (comments++), J. Gaspar (jag@isr.ist.utl.pt)
%

if isstruct(xz),
   %
   % allow input arguments to be specified in a struct
   % (as given by load_data.m)
   %
   mirror_vfield(xz.xz_mirror, xz.camera_to_mirror_offset)
   return;
end

% define data in relation to the camera projection centre
%
x2= camCentre2Mirror-xz(:,2);

figure;
subplot(121)
%
% show mirror profile
%
h= plot(xz(:,1),x2, '-'); set(h,'LineWidth',2)
%
hold on;
h= plot(-xz(:,1),x2, ['k' '-']); set(h,'LineWidth',2)
plot([-xz(end,1) 0 xz(end,1)], [x2(end) 0 x2(end)], 'y' )
draw_camera;
hold off;
%
axis('xy','equal')
title('Mirror profile (mm)')
%
beta= 90 - atan2(abs(x2(end)),abs(xz(end,1)))*180/pi;
xlabel(['Required lens vfield: ' num2str(round(2*beta)) 'deg'])

% show local slope
%
%subplot(132)
%plot(xz(1:(end-1),1),atan(diff(xz(:,2))./diff(xz(:,1)))*180/pi, '.-');
%title('Mirror local slope (deg)')

% show view angle
%
subplot(122)
beta= atan( xz(:,1) ./ x2 );
alfa= atan( diff(xz(:,2)) ./ diff(xz(:,1)) );
view_angle= (beta(1:(end-1))-2*alfa)*180/pi;
plot(xz(1:(end-1),1), view_angle, '.-');
%
title('View angle (deg) vs mirror radius (mm)')
xlabel(sprintf('Min,Max v. angles: %.1f,%.1f deg', min(view_angle), max(view_angle)))

return; % end of function


% ---------------- aux fn:
%
function draw_camera
%
r= 15; % mm
y1= r; y2= -r;
draw_rect( -r,y1, r,y2 )
y1= y2; y2= y2-r/4;
draw_rect( -r/2, y1, r/2, y2 )
y1= y2; y2= y2-2*r;
draw_rect( -r*1.5, y1, r*1.5, y2 )


% ---------------- aux fn:
%
function draw_rect(x1,y1,x2,y2)
%
% xy1 is top-left, xy2 is bottom-right
%
R= [x1 y1; x2 y1; x2 y2; x1 y2; x1 y1];
h= plot(R(:,1),R(:,2),'k');
set(h,'LineWidth',2)
