function mirror_vfield_gui(cmd, h0)
%
% function mirror_vfield_gui
% ==========================
%
% Show the vertical view angle of an omnidirectional vision system
% combining a convex mirror and a pin-hole camera.
%
% The profile of the mirror is described as a collection of
% points. This collection of points may be loaded from one of
% three alternative sources:
%   - a matlab mat file
%   - a text file
%   - global vars
%
% See in the following the details on these formats.
%
% 
% Mirror Profile specified in a matlab mat file
% ---------------------------------------------
% 
% The mat file must contain two variables named
% "xz_mirror" (Nx2) and "camera_to_mirror_offset" (scalar).
% 
% The "xz_mirror" variable specifies the mirror profile according
% to the following co-ordinate system (the first column contain
% the x values).
% 
%             z
%           ^
%      \    |    /
%       \   |   /
%         . | .
%     ------+-------> x
%           |O
%           |                                  --
%           |         camera_to_mirror_offset= CO
%           |
%           |
%           + C
%           |
% 
% 
% Mirror Profile specified in a text file
% ---------------------------------------
% 
% This type of input is in accordance to the output of the mirror
% design package "MixedMirrorDesign".
% 
% The text file is expected to have a name in the form "*_lin.txt" or
% "*_log.txt". It contains two columns, the first with x values
% and the second with z values as defined in the co-ordinate system
% drawn in following picture. Each line is ended with a ";".
% 
% There is expected to exist also the file "*_params.m". It contains
% a line defining the "camera_to_mirror_offset" variable in matlab's
% syntax.
% 
%             z
%           ^
%           |
%           + C
%           |                                  --
%           |         camera_to_mirror_offset= CO
%           |
%         .-+-.
%        /  |  \
%       .   |   .
%    -------+--------> x
%           |O
% 
% 
% Mirror Profile specified with matlab workspace vars
% ---------------------------------------------------
% 
% Use the same kind of data as described for the input
% based on mat files, i.e. define variables "xz_mirror"
% and "camera_to_mirror_offset" with the same meanings.
% 
%
% See also LOAD_DATA, MIRROR_VFIELD
%

%
%   Instituto Superior Tcnico / Instituto de Sistemas e Robtica
%           Computer Vision Lab: OMNIVIEWS PROJECT
%           http://www.isr.ist.utl.pt/labs/vislab/
%
% 10/10/01, J. Gaspar (jag@isr.ist.utl.pt)
%

if nargin<2,
   %
   % launch the graphical interface
   %
   if ~ishandle(101),
      draw_buttons;
   else
      s= get(101,'Name');
      if ~strcmp(s,'mirror vfield gui'),
         draw_buttons;
      end
   end
else
   %
   % execute commands as required from
   %  the graphical interface
   %
   buttons_cmd(cmd,h0)
end
return; % end of function


% -------------------------------- aux fn
%
function radio_select(n, h0)
%
h1 = findobj(h0, 'Tag', 'Radiobutton1');
h2 = findobj(h0, 'Tag', 'Radiobutton2');
h3 = findobj(h0, 'Tag', 'Radiobutton3');
set(h1,'Value',0);
set(h2,'Value',0);
set(h3,'Value',0);
if n==3,
   set(h3,'Value',1);
elseif n==2,
   set(h2,'Value',1);
else
   set(h1,'Value',1);
end


% -------------------------------- aux fn
%
function n= radio_selected(h0)
%
h1 = findobj(h0, 'Tag', 'Radiobutton1');
h2 = findobj(h0, 'Tag', 'Radiobutton2');
h3 = findobj(h0, 'Tag', 'Radiobutton3');

if get(h3,'Value'),
   n=3;
elseif get(h2,'Value'),
   n=2;
else
   n=1;
end


% -------------------------------- aux fn
%
function show_mirror(h0)

n= radio_selected(h0);
switch n,
   
case 3,
   h= findobj(h0, 'Tag', 'EditText_var1');
   h2= findobj(h0, 'Tag', 'EditText_var2');
   s= get(h,'String');
   s2= get(h2,'String');
   str= ['global ' s ' ' s2 '; a___=' s '; b___=' s2 ';'];
   eval(str)
   if isempty(a___),
      error(['Global "' s '" is empty.'])
   end
   if isempty(b___),
      error(['Global "' s2 '" is empty.'])
   end
   if size(a___,2)~=2,
      error(['Global var "' s '" has not 2 columns.'])
   end
   mirror_vfield( load_data(2, a___,b___) );
   
case 2,
   h= findobj(h0, 'Tag', 'EditText_txt_fname');
   h2= findobj(h0, 'Tag', 'EditText_params_fname');
   s= get(h2,'String');
   if isempty(s) | strcmp(s, '-- guess from the txt file --'),
      mirror_vfield( load_data(1, get(h,'String')) );
   else
      mirror_vfield( load_data(1, get(h,'String'), s) );
   end
   
otherwise, % n==1
   h= findobj(h0, 'Tag', 'EditText_mat_fname');
   mirror_vfield( load_data(0, get(h,'String')) );
   
end


% -------------------------------- aux fn
%
function get_fname(n, h0)
%

if n==3,
   [fname, dname]= uigetfile('*.m', 'Select m file with the mirror parameters');
   h = findobj(h0, 'Tag', 'EditText_params_fname');
   set(h, 'String', [dname fname]);
   
elseif n==2,
   [fname, dname]= uigetfile('*.txt', 'Select text file with the mirror points');
   h = findobj(h0, 'Tag', 'EditText_txt_fname');
   set(h, 'String', [dname fname]);
   
else % n==1
   [fname, dname]= uigetfile('*.mat', 'Select mat file with the mirror points');
   h = findobj(h0, 'Tag', 'EditText_mat_fname');
   set(h, 'String', [dname fname]);
end


% -------------------------------- aux fn
%
function buttons_cmd(cmd, h0)
%
switch cmd,
   
case 'radio_select1',
   radio_select(1, h0);
case 'radio_select2',
   radio_select(2, h0);
case 'radio_select3',
   radio_select(3, h0);
   
case 'browse_mat_fname',
   get_fname(1, h0);
   radio_select(1, h0);
case 'browse_txt_fname',
   get_fname(2, h0);
   radio_select(2, h0);
case 'browse_params_fname',
   get_fname(3, h0);
   radio_select(2, h0);
   
case 'show_mirror',
   show_mirror(h0);

end


% -------------------------------- aux fn
%
function h0= draw_buttons
%

h0= figure(101);
%h0 = figure('Color',[0.8 0.8 0.8], ...
set(h0, ...
   ... %'Colormap',mat0, ...
   'Name', 'mirror vfield gui', ...
	... %'FileName','U:\Omniviews\mirrors\vfield\vf_tmp.m', ...
	'PaperPosition',[18 180 576 432], ...
	'PaperUnits','points', ...
	'Position',[183    25   501   272], ...%[342 245 501 272], ...
	'Tag','Fig1', ...
	'ToolBar','none');
h1 = uicontrol('Parent',h0, ...
	'Units','points', ...
	'BackgroundColor',[0.752941176470588 0.752941176470588 0.752941176470588], ...
	'ListboxTop',0, ...
	'Position',[39.75 108 47.25 17.25], ...
	'String','params file', ...
	'Style','text', ...
	'Tag','StaticText1');
h1 = uicontrol('Parent',h0, ...
	'Units','points', ...
	'BackgroundColor',[0.752941176470588 0.752941176470588 0.752941176470588], ...
	'ListboxTop',0, ...
	'Position',[39.75 78.75 96.75 16.5], ...
	'String','Global vars: xz', ...
	'Style','text', ...
	'Tag','StaticText2');
h1 = uicontrol('Parent',h0, ...
	'Units','points', ...
	'BackgroundColor',[1 1 1], ...
	'ListboxTop',0, ...
	'Position',[99 107.25 195 18], ...
	'String','-- guess from the txt file --', ...
	'Style','edit', ...
	'Tag','EditText_params_fname');
h1 = uicontrol('Parent',h0, ...
	'Units','points', ...
	'BackgroundColor',[1 1 1], ...
	'ListboxTop',0, ...
	'Position',[185.25 78 109.5 18], ...
	'String','xz_mirror', ...
	'Style','edit', ...
	'Tag','EditText_var1');
h1 = uicontrol('Parent',h0, ...
   'Units','points', ...
   'Callback', mkcallbackstr( 'browse_params_fname', h0 ), ...
	'ListboxTop',0, ...
	'Position',[309.75 106.5 47.25 18], ...
	'String','browse', ...
	'Tag','Pushbutton_params_fname');
h1 = uicontrol('Parent',h0, ...
	'Units','points', ...
	'BackgroundColor',[0.752941176470588 0.752941176470588 0.752941176470588], ...
   'Callback', mkcallbackstr( 'show_mirror', h0 ), ...
	'ListboxTop',0, ...
	'Position',[102.75 14.25 78 25.5], ...
	'String','Show mirror', ...
	'Tag','Pushbutton_show_mirror');
h1 = uicontrol('Parent',h0, ...
	'Units','points', ...
	'BackgroundColor',[0.752941176470588 0.752941176470588 0.752941176470588], ...
	'Callback','helpwin mirror_vfield_gui', ...
	'ListboxTop',0, ...
	'Position',[201 15 62.25 24.75], ...
	'String','Help', ...
	'Tag','Pushbutton_help');
h1 = uicontrol('Parent',h0, ...
   'Units','points', ...
   'Callback', mkcallbackstr( 'browse_txt_fname', h0 ), ...
	'ListboxTop',0, ...
	'Position',[309.75 131.25 47.25 17.25], ...
	'String','browse', ...
	'Tag','Pushbutton_txt_fname');
h1 = uicontrol('Parent',h0, ...
   'Units','points', ...
   'Callback', mkcallbackstr( 'browse_mat_fname', h0 ), ...
	'ListboxTop',0, ...
	'Position',[309.75 162 47.25 18], ...
	'String','browse', ...
	'Tag','Pushbutton_mat_fname');
h1 = uicontrol('Parent',h0, ...
	'Units','points', ...
	'BackgroundColor',[1 1 1], ...
	'ListboxTop',0, ...
	'Position',[99.75 130.5 195 18], ...
	'String','demo2_lin.txt', ...
	'Style','edit', ...
	'Tag','EditText_txt_fname');
h1 = uicontrol('Parent',h0, ...
	'Units','points', ...
	'BackgroundColor',[1 1 1], ...
	'ListboxTop',0, ...
	'Position',[99 162 195 18], ...
	'String','demo1.mat', ...
	'Style','edit', ...
	'Tag','EditText_mat_fname');
h1 = uicontrol('Parent',h0, ...
	'Units','points', ...
	'BackgroundColor',[0.752941176470588 0.752941176470588 0.752941176470588], ...
	'ListboxTop',0, ...
	'Position',[35.25 132 54.75 15.75], ...
	'String','txt file', ...
	'Style','text', ...
	'Tag','StaticText2');
h1 = uicontrol('Parent',h0, ...
	'Units','points', ...
	'BackgroundColor',[1 1 1], ...
	'ListboxTop',0, ...
	'Position',[185.25 57 109.5 18], ...
	'String','camera_to_mirror_offset', ...
	'Style','edit', ...
	'Tag','EditText_var2');
h1 = uicontrol('Parent',h0, ...
	'Units','points', ...
	'BackgroundColor',[0.752941176470588 0.752941176470588 0.752941176470588], ...
	'ListboxTop',0, ...
	'Position',[40.5 55.5 96.75 16.5], ...
	'String','camera to mirror offset', ...
	'Style','text', ...
	'Tag','StaticText2');
h1 = uicontrol('Parent',h0, ...
	'Units','points', ...
	'BackgroundColor',[0.752941176470588 0.752941176470588 0.752941176470588], ...
   'Callback', mkcallbackstr( 'radio_select1', h0 ), ...
	'ListboxTop',0, ...
	'Position',[15.75 163.5 19.5 15], ...
	'Style','radiobutton', ...
	'Tag','Radiobutton1', ...
	'Value',1);
h1 = uicontrol('Parent',h0, ...
	'Units','points', ...
	'BackgroundColor',[0.752941176470588 0.752941176470588 0.752941176470588], ...
   'Callback', mkcallbackstr( 'radio_select2', h0 ), ...
	'ListboxTop',0, ...
	'Position',[15.75 132 20.25 15.75], ...
	'Style','radiobutton', ...
	'Tag','Radiobutton2');
h1 = uicontrol('Parent',h0, ...
	'Units','points', ...
	'BackgroundColor',[0.752941176470588 0.752941176470588 0.752941176470588], ...
   'Callback', mkcallbackstr( 'radio_select3', h0 ), ...
	'ListboxTop',0, ...
	'Position',[15.75 79.5 24.75 15], ...
	'Style','radiobutton', ...
	'Tag','Radiobutton3');
h1 = uicontrol('Parent',h0, ...
	'Units','points', ...
	'BackgroundColor',[0.752941176470588 0.752941176470588 0.752941176470588], ...
	'ListboxTop',0, ...
	'Position',[35.25 162.75 53.25 15.75], ...
	'String','mat file', ...
	'Style','text', ...
	'Tag','StaticText2');
if nargout > 0, fig = h0; end


% ---------- aux fn:
%
function s2= mkcallbackstr(s,h0)
%
s2= ['mirror_vfield_gui(''' s ''',' num2str(h0) ')'];

